@@ -1,8 +1,12 @@
 """red, mil, blue 3가지 분야에서 자료를 계산하여 리턴하는 함수 모음
 """
 from .db import CorpsEval
-from db_hj3415 import mongo
+from db_hj3415 import mongo, setting
 from util_hj3415 import utils
+from krx_hj3415 import krx
+import pandas as pd
+import time
+from multiprocessing import Process, cpu_count, Queue
 
 import logging
 
@@ -236,4 +240,121 @@ def growth(code: str) -> dict:
 '영업활동으로인한현금흐름'
 '재무활동으로인한현금흐름'
 'ROIC'
-"""
\ No newline at end of file
+"""
+
+
+def _make_df_part(codes: list, q):
+    def make_record(c: str) -> dict:
+        # 장고에서 사용할 eval 테이블을 만들기 위해 각각의 레코드를 구성하는 함수
+        c101 = mongo.C101(code=c).get_recent()
+
+        red_dict = red(code=c)
+        mil_dict = mil(code=c)
+        growth_dict = growth(code=c)
+
+        mil_date = mil_dict['date']
+        red_date = red_dict['date']
+        growth_date = growth_dict['date']
+
+        return {
+            'code': c101['코드'],
+            '종목명': c101['종목명'],
+            '주가': utils.to_int(c101['주가']),
+            'PER': utils.to_float(c101['PER']),
+            'PBR': utils.to_float(c101['PBR']),
+            '시가총액': utils.to_float(c101['시가총액']),
+            'RED': utils.to_int(red_dict['red_price']),
+            '주주수익률': utils.to_float(mil_dict['주주수익률']),
+            '이익지표': utils.to_float(mil_dict['이익지표']),
+            'ROIC': utils.to_float(mil_dict['투자수익률']['ROIC']),
+            'ROE': utils.to_float(mil_dict['투자수익률']['ROE']),
+            'PFCF': utils.to_float(CorpsEval.get_recent(mil_dict['가치지표']['PFCF'])[1]),
+            'PCR': utils.to_float(CorpsEval.get_recent(mil_dict['가치지표']['PCR'])[1]),
+            '매출액증가율': utils.to_float(growth_dict['매출액증가율'][0]),
+            'date': list(set(mil_date + red_date + growth_date))
+        }
+
+    t = len(codes)
+    d = []
+    for i, code in enumerate(codes):
+        print(f'{i+1}/{t} {code}')
+        try:
+            d.append(make_record(code))
+        except:
+            logger.error(f'error on {code}')
+            continue
+    df = pd.DataFrame(d)
+    logger.info(df)
+    q.put(df)
+
+
+def get_df() -> pd.DataFrame:
+    """ 멀티프로세싱을 사용하여 전체 종목의 eval 을 데이터프레임으로 만들어 반환
+
+    """
+    def _code_divider(entire_codes: list) -> tuple:
+        # 전체 종목코드를 리스트로 넣으면 cpu 코어에 맞춰 나눠준다.
+        # https://stackoverflow.com/questions/19086106/how-to-utilize-all-cores-with-python-multiprocessing
+        def _split_list(alist, wanted_parts=1):
+            # 멀티프로세싱할 갯수로 리스트를 나눈다.
+            # https://www.it-swarm.dev/ko/python/%EB%8D%94-%EC%9E%91%EC%9D%80-%EB%AA%A9%EB%A1%9D%EC%9C%BC%EB%A1%9C-%EB%B6%84%ED%95%A0-%EB%B0%98%EC%9C%BC%EB%A1%9C-%EB%B6%84%ED%95%A0/957910776/
+            length = len(alist)
+            return [alist[i * length // wanted_parts: (i + 1) * length // wanted_parts]
+                    for i in range(wanted_parts)]
+
+        core = cpu_count()
+        print(f'Get number of core for multiprocessing : {core}')
+        n = core - 1
+        if len(entire_codes) < n:
+            n = len(entire_codes)
+        print(f'Split total {len(entire_codes)} codes by {n} parts ...')
+        divided_list = _split_list(entire_codes, wanted_parts=n)
+        return n, divided_list
+
+    print(setting.load())
+    codes = mongo.Corps.get_all_corps()
+
+    print('*' * 25, f"Eval all using multiprocess", '*' * 25)
+    print(f'Total {len(codes)} items..')
+    logger.info(codes)
+    n, divided_list = _code_divider(codes)
+
+    start_time = time.time()
+    q = Queue()
+    ths = []
+    for i in range(n):
+        ths.append(Process(target=_make_df_part, args=(divided_list[i], q)))
+    for i in range(n):
+        ths[i].start()
+    df_list = []
+    for i in range(n):
+        df_list.append(q.get())
+    # 부분데이터프레임들을 하나로 합침
+    final_df = pd.concat(df_list, ignore_index=True)
+    for i in range(n):
+        ths[i].join()
+    print(f'Total spent time : {round(time.time() - start_time, 2)} sec.')
+    logger.debug(final_df)
+    return final_df
+
+
+def yield_valid_spac() -> tuple:
+    """
+    전체 스팩주의 현재가를 평가하여 2000원 이하인 경우 yield한다.
+
+    Returns:
+        tuple: (code, name, price)
+    """
+    codes = mongo.Corps.get_all_corps()
+    logger.info(f'len(codes) : {len(codes)}')
+    print('<<< Finding valuable SPAC >>>')
+    for i, code in enumerate(codes):
+        name = krx.get_name(code)
+        logger.info(f'code : {code} name : {name}')
+        if '스팩' in str(name):
+            logger.info(f'>>> spac - code : {code} name : {name}')
+            price, _, _ = utils.get_price_now(code=code)
+            if price <= 2000:
+                logger.warning(f'현재가:{price}')
+                print(f"code: {code} name: {name}, price: {price}")
+                yield code, name, price
